# Generated by Django 2.0.6 on 2019-02-19 21:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('whistle', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='notification',
            name='details',
            field=models.TextField(blank=True, default=''),
        ),
    ]
